package com.zhilian.resolver.model;
import com.zhilian.common.core.utils.SpringUtils;
import com.zhilian.common.resolver.domain.ResolverReportData;
import com.zhilian.resolver.service.ResolverEventService;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.time.Duration;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Properties;
import static com.zhilian.resolver.utils.ConvertUtils.hexStringToString;
import static com.zhilian.resolver.utils.ConvertUtils.parseVehicleData;

/**
  * @ClassName ModelsKafkaMessage
  * @Description 描述
  * @Author Can.J
  * @Date  2024/4/8
 */
@Component
@Slf4j
public class ModelsKafkaMessage {
    private static final String TOPIC_NAME = "test-topic";
    private static final String BOOTSTRAP_SERVERS = "10.10.25.5:9092";

    static ArrayList<String> stringEvents = new ArrayList<>() {
        {
            add("malfunction");     //故障事件
            add("event-realTimeTrajectory"); //实时轨迹数事件
            add("event-storage");      //存储服务事件
            add("geofence");        //电子围栏事件
        }
    };
    /**
     * 消费者配置
     * @return
     */
    @Scheduled(fixedDelay = 50)
    private static void consumerMessages() {
        Properties props = new Properties();
        props.put(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG, BOOTSTRAP_SERVERS);
        props.put(ConsumerConfig.GROUP_ID_CONFIG, "my-consumer-group");
        props.put(ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG, "org.apache.kafka.common.serialization.StringDeserializer");
        props.put(ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG, "org.apache.kafka.common.serialization.StringDeserializer");

        //创建消费者
        KafkaConsumer<String, String> consumer = new KafkaConsumer<>(props);

        try {

            //订阅主题
            consumer.subscribe(Collections.singletonList(TOPIC_NAME));

            //持续消费消息
            while (true) {
                ConsumerRecords<String, String> records = consumer.poll(Duration.ofMillis(100));
                records.forEach(record -> {
                    System.out.println("接收到的数据：" + record.value());
                    String str = hexStringToString(record.value());
                    List<ResolverReportData> resolverReportDataList = parseVehicleData(str);
                    for (ResolverReportData vehicleData : resolverReportDataList) {
                        log.info("解析到车辆数据：{}", vehicleData);
                            //str-->vehicleDataList-->vehicleData实例
                            log.info("解析到车辆数据：{}", vehicleData);
                            for (String stringEvent : stringEvents) {
                                ResolverEventService resolverEventService =SpringUtils.getBean(stringEvent);
                                resolverEventService.execute(vehicleData);

                        }

                    }
                });
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            consumer.close();
        }
    }


}
